# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'editwidget.ui'
#
# Created: Wed Apr 11 19:58:33 2012
#      by: PyQt4 UI code generator 4.8.5
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_EditWidget(object):
    def setupUi(self, EditWidget):
        EditWidget.setObjectName(_fromUtf8("EditWidget"))
        EditWidget.setWindowModality(QtCore.Qt.WindowModal)
        EditWidget.resize(796, 251)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(EditWidget.sizePolicy().hasHeightForWidth())
        EditWidget.setSizePolicy(sizePolicy)
        EditWidget.setWindowTitle(QtGui.QApplication.translate("EditWidget", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.verticalLayout = QtGui.QVBoxLayout(EditWidget)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.scrollArea = QtGui.QScrollArea(EditWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QtGui.QFrame.Plain)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 941, 55))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.pushButton = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton.setText(_fromUtf8(""))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-bold.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon)
        self.pushButton.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+B", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton.setObjectName(_fromUtf8("pushButton"))
        self.horizontalLayout_2.addWidget(self.pushButton)
        self.pushButton_2 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_2.setText(_fromUtf8(""))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-italic.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_2.setIcon(icon1)
        self.pushButton_2.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+I", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_2.setObjectName(_fromUtf8("pushButton_2"))
        self.horizontalLayout_2.addWidget(self.pushButton_2)
        self.pushButton_3 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_3.setText(_fromUtf8(""))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-underline.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_3.setIcon(icon2)
        self.pushButton_3.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+U", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_3.setObjectName(_fromUtf8("pushButton_3"))
        self.horizontalLayout_2.addWidget(self.pushButton_3)
        self.pushButton_4 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_4.setText(_fromUtf8(""))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-strikethrough.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_4.setIcon(icon3)
        self.pushButton_4.setObjectName(_fromUtf8("pushButton_4"))
        self.horizontalLayout_2.addWidget(self.pushButton_4)
        self.line = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line.setFrameShape(QtGui.QFrame.VLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.horizontalLayout_2.addWidget(self.line)
        self.pushButton_5 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_5.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_5.setText(_fromUtf8(""))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-color.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_5.setIcon(icon4)
        self.pushButton_5.setObjectName(_fromUtf8("pushButton_5"))
        self.horizontalLayout_2.addWidget(self.pushButton_5)
        self.pushButton_6 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_6.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_6.setText(_fromUtf8(""))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-fill-color.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_6.setIcon(icon5)
        self.pushButton_6.setObjectName(_fromUtf8("pushButton_6"))
        self.horizontalLayout_2.addWidget(self.pushButton_6)
        self.fontComboBox = QtGui.QFontComboBox(self.scrollAreaWidgetContents)
        self.fontComboBox.setMaximumSize(QtCore.QSize(120, 16777215))
        self.fontComboBox.setObjectName(_fromUtf8("fontComboBox"))
        self.horizontalLayout_2.addWidget(self.fontComboBox)
        self.comboFontSize = QtGui.QComboBox(self.scrollAreaWidgetContents)
        self.comboFontSize.setMaximumSize(QtCore.QSize(50, 16777215))
        self.comboFontSize.setEditable(True)
        self.comboFontSize.setObjectName(_fromUtf8("comboFontSize"))
        self.horizontalLayout_2.addWidget(self.comboFontSize)
        self.line1 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line1.setFrameShape(QtGui.QFrame.VLine)
        self.line1.setFrameShadow(QtGui.QFrame.Sunken)
        self.line1.setObjectName(_fromUtf8("line1"))
        self.horizontalLayout_2.addWidget(self.line1)
        self.pushButton_7 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_7.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_7.setText(_fromUtf8(""))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-left.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_7.setIcon(icon6)
        self.pushButton_7.setObjectName(_fromUtf8("pushButton_7"))
        self.horizontalLayout_2.addWidget(self.pushButton_7)
        self.pushButton_8 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_8.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_8.setText(_fromUtf8(""))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-center.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_8.setIcon(icon7)
        self.pushButton_8.setObjectName(_fromUtf8("pushButton_8"))
        self.horizontalLayout_2.addWidget(self.pushButton_8)
        self.pushButton_9 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_9.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_9.setText(_fromUtf8(""))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-right.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_9.setIcon(icon8)
        self.pushButton_9.setObjectName(_fromUtf8("pushButton_9"))
        self.horizontalLayout_2.addWidget(self.pushButton_9)
        self.pushButton_10 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_10.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_10.setText(_fromUtf8(""))
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-fill.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_10.setIcon(icon9)
        self.pushButton_10.setObjectName(_fromUtf8("pushButton_10"))
        self.horizontalLayout_2.addWidget(self.pushButton_10)
        self.line_3 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_3.setFrameShape(QtGui.QFrame.VLine)
        self.line_3.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_3.setObjectName(_fromUtf8("line_3"))
        self.horizontalLayout_2.addWidget(self.line_3)
        self.comboInsertList = QtGui.QComboBox(self.scrollAreaWidgetContents)
        self.comboInsertList.setObjectName(_fromUtf8("comboInsertList"))
        self.horizontalLayout_2.addWidget(self.comboInsertList)
        self.pushButton_11 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_11.setEnabled(False)
        self.pushButton_11.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_11.setText(_fromUtf8(""))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-indent-more.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_11.setIcon(icon10)
        self.pushButton_11.setObjectName(_fromUtf8("pushButton_11"))
        self.horizontalLayout_2.addWidget(self.pushButton_11)
        self.pushButton_12 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_12.setEnabled(False)
        self.pushButton_12.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_12.setText(_fromUtf8(""))
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-indent-less.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_12.setIcon(icon11)
        self.pushButton_12.setObjectName(_fromUtf8("pushButton_12"))
        self.horizontalLayout_2.addWidget(self.pushButton_12)
        self.line_2 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_2.setFrameShape(QtGui.QFrame.VLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.horizontalLayout_2.addWidget(self.line_2)
        self.pushButton_13 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_13.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_13.setText(_fromUtf8(""))
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-horizontal-rule.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_13.setIcon(icon12)
        self.pushButton_13.setObjectName(_fromUtf8("pushButton_13"))
        self.horizontalLayout_2.addWidget(self.pushButton_13)
        self.pushButton_14 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_14.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_14.setToolTip(QtGui.QApplication.translate("EditWidget", "Insert External Link", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_14.setText(_fromUtf8(""))
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-link.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_14.setIcon(icon13)
        self.pushButton_14.setObjectName(_fromUtf8("pushButton_14"))
        self.horizontalLayout_2.addWidget(self.pushButton_14)
        self.pushButton_15 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_15.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_15.setText(_fromUtf8(""))
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-image.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_15.setIcon(icon14)
        self.pushButton_15.setObjectName(_fromUtf8("pushButton_15"))
        self.horizontalLayout_2.addWidget(self.pushButton_15)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.textEdit = QtGui.QTextEdit(EditWidget)
        self.textEdit.setLineWrapMode(QtGui.QTextEdit.WidgetWidth)
        self.textEdit.setObjectName(_fromUtf8("textEdit"))
        self.verticalLayout.addWidget(self.textEdit)
        self.verticalLayout.setStretch(1, 1)

        self.retranslateUi(EditWidget)
        QtCore.QObject.connect(self.pushButton_2, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotItalic)
        QtCore.QObject.connect(self.pushButton_3, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotUnderline)
        QtCore.QObject.connect(self.pushButton_4, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotStrikeout)
        QtCore.QObject.connect(self.fontComboBox, QtCore.SIGNAL(_fromUtf8("currentFontChanged(QFont)")), self.textEdit.setCurrentFont)
        QtCore.QObject.connect(self.textEdit, QtCore.SIGNAL(_fromUtf8("textChanged()")), EditWidget.slotTextChanged)
        QtCore.QObject.connect(self.pushButton_5, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotTextColor)
        QtCore.QObject.connect(self.pushButton_6, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotFillColor)
        QtCore.QObject.connect(self.comboFontSize, QtCore.SIGNAL(_fromUtf8("currentIndexChanged(QString)")), EditWidget.slotFontSize)
        QtCore.QObject.connect(self.pushButton_7, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignLeft)
        QtCore.QObject.connect(self.pushButton_8, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignCenter)
        QtCore.QObject.connect(self.pushButton_9, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignRight)
        QtCore.QObject.connect(self.pushButton_10, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignFill)
        QtCore.QObject.connect(self.textEdit, QtCore.SIGNAL(_fromUtf8("cursorPositionChanged()")), EditWidget.slotCursorPositionChanged)
        QtCore.QObject.connect(self.pushButton, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotBold)
        QtCore.QObject.connect(self.comboInsertList, QtCore.SIGNAL(_fromUtf8("currentIndexChanged(int)")), EditWidget.slotInsertList)
        QtCore.QObject.connect(self.pushButton_11, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotIndent)
        QtCore.QObject.connect(self.pushButton_12, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotUnindent)
        QtCore.QObject.connect(self.pushButton_13, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertHR)
        QtCore.QObject.connect(self.pushButton_14, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertLink)
        QtCore.QObject.connect(self.pushButton_15, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertImage)
        QtCore.QMetaObject.connectSlotsByName(EditWidget)

    def retranslateUi(self, EditWidget):
        pass

import rprj_rc
